/** 
 * A symbol table implementation 
 */
package minijava;

import minijava.analysis.*;
import minijava.node.*;
import java.util.*;

public class SymbolTable{
    private Hashtable hashtable;
    
    // Default constructor
    public SymbolTable() 
    {
	hashtable = new Hashtable();
    }
    
    public boolean addClass(String id, String parent) 
    {
	if (containsClass(id)) 
	    return false;	    
	else 
	    hashtable.put(id, new Class(id, parent));
	return true;	    
    }
    
    public Class getClass(String id)
    {
	if(containsClass(id)) 
	    return (Class)hashtable.get(id);	    
	else 
	    return null;
    }
    
    public boolean containsClass(String id)
    {
	return hashtable.containsKey(id);
    }
    
    public Node getVarType(Method m, Class c, String id)
    {
	if(m != null) {
	    if(m.getVariable(id) != null) {
		return m.getVariable(id).getType();
	    }
	    if(m.getParameter(id) != null){
		return m.getParameter(id).getType();
	    }
	}
	
	while(c != null) {
	    if(c.getVariable(id) != null) {
		return c.getVariable(id).getType();
	    }
	    else {
		if(c.getParent() == null) {
		    c = null;
		}
		else {
		    c = getClass(c.getParent());
		}
	    }
	}
      
      
	System.out.println("Variable " + id  + " not defined in current scope");
	System.exit(0);
	return null;
    }
    
    public Method getMethod(String id, String classScope)
    {
	if (getClass(classScope)==null){
	    System.out.println("Class " + classScope + " not defined");  
	    System.exit(0);
	}

	Class c = getClass(classScope);
	while(c != null) {
	    if(c.getMethod(id) != null) {
		return c.getMethod(id);
	    }
	    else {
		if(c.getParent() == null) {
		    c = null;
		}
		else {
		    c = getClass(c.getParent());
		}
	    }
	}

	
	System.out.println("Method " + id + " not defined in class " + classScope);
	
	System.exit(0);
	return null;
    }

    public Node getMethodType(String id, String classScope) 
    {
	if(getClass(classScope)==null) {
	    System.out.println("Class " + classScope + " not defined");  
	    System.exit(0);
	}
	
	Class c = getClass(classScope);
	while(c != null) {
	    if(c.getMethod(id) != null) {
		return c.getMethod(id).getType();
	    }
	    else {
		if(c.getParent() == null) {
		    c = null;
		}
		else {
		    c = getClass(c.getParent());
		}
	    }
	}
	
	System.out.println("Method " + id + " not defined in class " + classScope);	
	System.exit(0);
	return null;
    }

    public boolean compareTypes(Node t1, Node t2)
    {
	
	if (t1 == null || t2 == null) return false;
	
	if (t1 instanceof AIntegerType && t2 instanceof AIntegerType)
	    return true;
	if (t1 instanceof ABooleanType && t2 instanceof ABooleanType)
	    return true;
	if (t1 instanceof AArrayTypeType && t2 instanceof AArrayTypeType)
	    return true;
	if (t1 instanceof AIdentifierType && t2 instanceof AIdentifierType){
	    AIdentifierType i1 = (AIdentifierType)t1;
	    AIdentifierType i2 = (AIdentifierType)t2;
	    
	    /*
	    Class c = getClass(i2.toString());
	    while(c != null) {
		if (i1.s.equals(c.getId())) return true;
		else {
		    if(c.getParent() == null) return false;
		    c = getClass(c.getParent());
		}
	    }
	    */
	}
	return false;	
    }
}
